package JHamlConverter;

import java.io.*;
import java.util.*;

import static JHamlConverter.DebugAndTest.debug;
import static JHamlConverter.DirectMethods.*;

public class HelperMethods {

    /**
     * Method that makes a Arraylist<String> out of a file
     * @param filepath - a string with the filepath
     * @return list - Arraylist<String>
     */
    public static ArrayList<String> makeLists(String filepath) throws IOException {
        ArrayList<String> list = PopulateList(filepath);
        PopulateList(filepath);

        return list;
    }

    /**
     * Method that fetches the end index value of the starter tag
     * @param line - a string where we need to find the correct index of where the tag should end
     * @param indentation - integer value of the indentation of current line
     * @return endTagIndex - integer value that represents the end index of the begining-tag
     */
    public static int getEndTagIndex(String line, int indentation) {
        int endTagIndex;
        int spaceCase = line.substring(indentation).replaceAll(" ", "@").indexOf("@"); // .indexOf didn't work with " " so replaced all " " with '@'

        if(line.indexOf('(') > 0) {
            endTagIndex = line.indexOf('(');
        }
        else if (line.indexOf('.') > 0) {
            endTagIndex = line.indexOf('.');
        }
        else if (line.indexOf('#') > 0) {
            endTagIndex = line.indexOf('#');
        }
        else if (spaceCase > 0) {
            endTagIndex = spaceCase + indentation;
        }
        else if (line.indexOf('/') > 0) {
            endTagIndex = line.indexOf('/');
        }
        else {
            endTagIndex = line.length();
        }
        return endTagIndex;
    }
    /**
     * Method that gets the index of the current headParent to use in getChildrenElements()
     * @param lines - filepath to read a specific document
     * @param parentLine - current parent line
     * @return parentLine - integer value representing index number of the current headParent
     */
    public static int getParentLine(ArrayList<String> lines, String parentLine, int lineCounter){
        int counter = 0;
        int parentIndex = 0;

        if(!(parentLine.charAt(parentLine.length()-1) == '%')) {
            while (counter < lines.size()) {

                if (lines.get(counter).startsWith(parentLine)) {

                    parentIndex = counter;
                    break;
                }
                counter++;
            }
        }
        else{
            parentIndex = lineCounter-1;
        }
        return parentIndex;
    }

    /**
     * Recursive method that will check if a child has another child
     * @param childLine The current child being processed
     * @param parentLine Parent line of current child
     * @param linecounter Current line index
     * @param Jhaml The to be converted document represented by lines (each index = 1 line (Arraylist<String>))
     * @param writer Filewriter that will write the lines to the html file
     * @return update linecounter
     * @throws IOException
     */
    public static int parseTags(String childLine, String parentLine, int linecounter, ArrayList<String> Jhaml,FileWriter writer) throws IOException {
        char startingChar = getFirstChar(childLine);
        String [] tagParts;
        int indentation = indexOfChar(childLine);
        int childrenElements = getChildrenElements(Jhaml, childLine, linecounter);

        if (childrenElements > 0 && startingChar == '%' || startingChar == '#' || startingChar == '.') {

            tagParts = createTagDetails(childLine,indentation,writer,"depth");
            linecounter++;
            parseTags(Jhaml.get(linecounter), childLine, linecounter, Jhaml ,writer);
            linecounter += childrenElements;
            writer.write(createSpace(indentation) + makeEndTag(tagParts[0]) + "\n");
        }
        else {

            int nextlineIndent = 0;
            if(linecounter+1 < Jhaml.size()){

                nextlineIndent = indexOfChar(Jhaml.get(linecounter+1));
            }

            if(indentation > 4 && nextlineIndent == indentation) {

                while(indentation == nextlineIndent){

                    nextlineIndent = indexOfChar(Jhaml.get(linecounter+1));

                    if(nextlineIndent > indentation) {

                        createTagDetails(childLine, indentation, writer, "depth");
                        linecounter++;
                    }
                    else{
                        createTagDetails(childLine, indentation, writer, "");
                        linecounter++;
                    }
                    childLine = Jhaml.get(linecounter);

                    if(nextlineIndent > indentation){

                        String newparentLine = childLine;
                        childLine = Jhaml.get(linecounter);
                        parseTags(childLine,newparentLine,linecounter,Jhaml,writer);
                        writer.write(createSpace(indentation) + makeEndTag( "<" + childLine.substring(nextlineIndent+1,getEndTagIndex(childLine,nextlineIndent))) + "\n");
                    }
                }
            }
            else{
                createTagDetails(childLine, indentation, writer, "");
                linecounter++;
            }
        }
        return linecounter;
    }

    /**
     * Method that creates correct indentation
     * @param amountOfSpace - integer value that represents the amountOfSpace that needs to be created for indentation
     * @return space - String with indentation
     */
    public static String createSpace(int amountOfSpace) {
        String space = "";

        for (int i = 0;i < amountOfSpace;i++) {
            space += " ";
        }

        return space;
    }

    /**
     *
     * @param line
     * @return
     */
    public static char getFirstChar(String line){
        int indentation = indexOfChar(line);
        return line.charAt(indentation);
    }

    /**
     *
     * @param tag
     * @return
     */
    public static String makeEndTag(String tag){
       return tag.charAt(0) + "/" + tag.substring(1) + ">";
    }

    /**
     *
     * @return
     */
    public static ArrayList<String> makeIgnoreList(){
        ArrayList<String> ignoreList = new ArrayList<>();
        ignoreList.add("<area>");
        ignoreList.add("<base>");
        ignoreList.add("<br>");
        ignoreList.add("<col>");
        ignoreList.add("<command>");
        ignoreList.add("<embed>");
        ignoreList.add("<hr>");
        ignoreList.add("<img>");
        ignoreList.add("<input>");
        ignoreList.add("<keygen>");
        ignoreList.add("<link>");
        ignoreList.add("<meta>");
        ignoreList.add("<param>");
        ignoreList.add("<source>");
        ignoreList.add("<track>");
        ignoreList.add("<wbr>");
    return ignoreList;
    }

    /**
     * Method that prints the converted lines to the html file
     * @param indentation - integer value of the indentation of current line
     * @param tag - tag of the current line without '>'
     * @param rest - 'elements' of the current tag
     * @param txt - 'values' of the current tag
     * @param writer - Filewriter that will write the lines to the html file
     */
    public static void writeToFile(int indentation, String tag, String rest, String txt, FileWriter writer, String type) throws IOException {
        ArrayList<String> ignorelist = makeIgnoreList();

        if (!tag.equals("")) {
            String completeTag = "";
            completeTag += createSpace(indentation) + tag;

            if (!rest.equals("")) {
                completeTag += " " + rest + ">";
            }
            else if (tag.charAt(0) == '%') {

            }
            else if (!tag.contains("<")) {

            }
            else {
                if(!tag.contains("<!--")){
                    completeTag += ">";
                }
            }

            if (!txt.equals("")) {
                completeTag += txt;
            }
            if (!(tag.charAt(0) == '%') && !ignorelist.contains(tag + ">") && !type.equals("depth")) {
                if(!tag.contains("<!--")){
                    completeTag += makeEndTag(tag);
                }
                else {
                    completeTag += " -->";
                }
            }
            writer.write(completeTag + "\n");
        }
    }
}
